package geniusweb.party.inform;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import geniusweb.issuevalue.Bid;

/**
 * Informs that the session is finished, and also contains the bid that was
 * agreed on.
 *
 */
public class Finished implements Inform {
	private final Bid agreedBid;

	/**
	 * The session finished (can be for any reason).
	 * 
	 * @param agreedBid the bid that was finally agreed on; null if no
	 *                  agreement.
	 */
	@JsonCreator
	public Finished(@JsonProperty("agreedBid") Bid agreedBid) {
		this.agreedBid = agreedBid;
	}

	public Bid getAgreement() {
		return agreedBid;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((agreedBid == null) ? 0 : agreedBid.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Finished other = (Finished) obj;
		if (agreedBid == null) {
			if (other.agreedBid != null)
				return false;
		} else if (!agreedBid.equals(other.agreedBid))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Finished[" + agreedBid + "]";
	}
}
