package geniusweb.party.inform;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import geniusweb.actions.PartyId;
import geniusweb.progress.Progress;
import geniusweb.references.ProfileRef;
import geniusweb.references.ProtocolRef;

/**
 * Informs a party about all settings for the upcoming negotiation session. This
 * should be sent to a party one time, so that the party knows the situation.
 *
 */
public class Settings implements Inform {
	private ProfileRef profile;
	private ProtocolRef protocol;
	private Progress progress;
	private PartyId id;

	@JsonCreator
	public Settings(@JsonProperty("id") PartyId id,
			@JsonProperty("profile") ProfileRef profile,
			@JsonProperty("protocol") ProtocolRef protocol,
			@JsonProperty("progress") Progress progress) {
		if (profile == null || protocol == null || progress == null) {
			throw new IllegalArgumentException(
					"party, profile, protocol and progress must be not null.");
		}
		this.profile = profile;
		this.protocol = protocol;
		this.progress = progress;
		this.id = id;
	}

	public ProfileRef getProfile() {
		return profile;
	}

	public ProtocolRef getProtocol() {
		return protocol;
	}

	public Progress getProgress() {
		return progress;
	}

	public PartyId getID() {
		return id;
	}

	@Override
	public String toString() {
		return "Settings[" + id + "," + profile + "," + protocol + ","
				+ progress + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((profile == null) ? 0 : profile.hashCode());
		result = prime * result
				+ ((progress == null) ? 0 : progress.hashCode());
		result = prime * result
				+ ((protocol == null) ? 0 : protocol.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Settings other = (Settings) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (profile == null) {
			if (other.profile != null)
				return false;
		} else if (!profile.equals(other.profile))
			return false;
		if (progress == null) {
			if (other.progress != null)
				return false;
		} else if (!progress.equals(other.progress))
			return false;
		if (protocol == null) {
			if (other.protocol != null)
				return false;
		} else if (!protocol.equals(other.protocol))
			return false;
		return true;
	}

}
