package geniusweb.party;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.mockito.internal.util.collections.Sets;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.party.Capabilities;
import geniusweb.references.ProtocolRef;
import tudelft.utilities.junit.GeneralTests;

public class CapabilitiesTest extends GeneralTests<Capabilities> {

	private Capabilities capabilities1, capabilities1a, capabilities2,
			capabilities3;
	private String asJson = "{\"protocols\":[\"ws:localhost/protocol1\"]}";

	@Before
	public void before() throws URISyntaxException {
		capabilities1 = new Capabilities(Sets
				.newSet(new ProtocolRef(new URI("ws:localhost/protocol1"))));
		capabilities1a = new Capabilities(Sets
				.newSet(new ProtocolRef(new URI("ws:localhost/protocol1"))));
		capabilities2 = new Capabilities(Sets
				.newSet(new ProtocolRef(new URI("ws:localhost/protocol2"))));
		capabilities3 = new Capabilities(Sets
				.newSet(new ProtocolRef(new URI("ws:localhost/protocol3"))));
	}

	@Override
	public List<List<Capabilities>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(capabilities1, capabilities1a),
				Arrays.asList(capabilities2), Arrays.asList(capabilities3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"Capabilities.*Protocols=.*ProtocolRef.*protocol1.*",
				"Capabilities.*Protocols=.*ProtocolRef.*protocol2.*",
				"Capabilities.*Protocols=.*ProtocolRef.*protocol3.*");
	}

	@Test(expected = IllegalArgumentException.class)
	public void nullTest() throws URISyntaxException {
		new Capabilities(null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(capabilities1);
		System.out.println(json);
		assertEquals(asJson, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		Capabilities p = jackson.readValue(asJson, Capabilities.class);
		System.out.println(p);
		assertEquals(capabilities1, p);
	}

}
