package geniusweb.party.inform;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.Accept;
import geniusweb.actions.PartyId;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.Value;
import geniusweb.party.inform.ActionDone;
import geniusweb.party.inform.Inform;
import tudelft.utilities.junit.GeneralTests;

public class ActionDoneTest extends GeneralTests<ActionDone> {

	private ActionDone finished1, finished1a, finished2;

	private String asJson = "{\"ActionDone\":{\"action\":{\"accept\":{\"actor\":\"me\",\"bid\":{\"issuevalues\":{\"issue3\":9012,\"issue2\":1,\"issue1\":\"b\"}}}}}}";

	private PartyId me = new PartyId("me");

	@Before
	public void before() throws URISyntaxException {
		final String ISSUE1 = "issue1";
		final String ISSUE2 = "issue2";
		final Value VALUE1 = new DiscreteValue("value1");
		final Value VALUE2 = new NumberValue("10");

		Map<String, Value> issuevalues = new HashMap<>();
		issuevalues.put(ISSUE1, new DiscreteValue("b"));
		issuevalues.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		issuevalues.put("issue3", new NumberValue(new BigDecimal("9012")));
		Bid bid = new Bid(issuevalues);

		Map<String, Value> issuevaluesb = new HashMap<>();
		issuevaluesb.put(ISSUE1, new DiscreteValue("b"));
		issuevaluesb.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		Bid bidb = new Bid(issuevaluesb);

		// bidc and bidd have values swapped, so different issuevalues.
		Map<String, Value> issuevaluesc = new HashMap<>();
		issuevaluesc.put(ISSUE1, VALUE1);
		issuevaluesc.put(ISSUE2, VALUE2);

		finished1 = new ActionDone(new Accept(me, bid));
		finished1a = new ActionDone(new Accept(me, bid));
		finished2 = new ActionDone(new Accept(me, bidb));
	}

	@Override
	public List<List<ActionDone>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(finished1, finished1a),
				Arrays.asList(finished2));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("ActionDone.*Accept.*", "ActionDone.*Accept.*");
	}

	@Test
	public void nullTest() throws URISyntaxException {
		new ActionDone(null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(finished1);
		System.out.println(json);
		assertEquals(asJson, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		ActionDone p = (ActionDone) jackson.readValue(asJson, Inform.class);
		System.out.println(p);
		assertEquals(finished1, p);
	}

}
